/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.inscripcionCedulas.mundo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Clase que representa un departamento.<br>
 * <b>inv:</b><br>
 * nombre != null && nombre != "". <br>
 * municipios != null. <br>
 */
public class Departamento
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Constante que representa la serializacin.
     */
    private static final long serialVersionUID = 1L;

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Nombre del departamento.
     */
    private String nombre;

    /**
     * Lista de municipios del departamento.
     */
    private List<Municipio> municipios;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Inicializa un departamento con la informacin dada por parmetro.<br>
     * <b>post: </b>El departamento se inicializ con el valor nombre dado por parmetro.<br>
     * La lista de municipios qued inicializada.
     * @param pNombre Nombre del departamento. pNombre != null && pNombre != "".
     */
    public Departamento( String pNombre )
    {
        nombre = pNombre;
        municipios = new ArrayList<Municipio>( );

        verificarInvariante( );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Retorna el nombre del departamento.
     * @return Nombre del departamento.
     */
    public String darNombre( )
    {
        return nombre;
    }

    /**
     * Retorna la lista de municipios del departamento.
     * @return Municipios del departamento.
     */
    public List<Municipio> darMunicipios( )
    {
        return municipios;
    }

    /**
     * Retorna el nmero de ciudadanos inscritos en el departamento.
     * @return Nmero de ciudadanos inscritos en el departamento.
     */
    public int darNumInscritos( )
    {
        int numInscritos = 0;
        for (Municipio municipio : municipios) {
        	numInscritos += municipio.darNumInscritos();
        }
        return numInscritos;
    }

    /**
     * Agrega un municipio al departamento.
     * pre: 	la lista de municipios se encuentra inicializada
     * @param pMunicipio Nombre del municipio que debe ser agregado. pMunicipio != null y pMunicipio != "".
     */
    public void agregarMunicipio( String pMunicipio )
    {
        Municipio m = new Municipio( pMunicipio );
        municipios.add( m );
        verificarInvariante( );
    }

    /**
     * Inscribe a un ciudadano en el puesto de votacin y municipio dados por parmetro. <br>
     * pre: 	la lista de municipios se encuentra inicializada
     * 			el municipio existe en el departamento
     * @param pMunicipio Nombre del municipio al que se va a inscribir el ciudadano. municipio != null y municipio != "".
     * @param pPuesto Nombre del puesto de votacin al que se va a inscribir el ciudadano. puesto != null y puesto != "".
     * @param pCiudadano Ciudadano a inscribir en el departamento. pCiudadano != null.
     */
    public void inscribirCiudadano( String pMunicipio, String pPuesto, Ciudadano pCiudadano )
    {
    	Municipio m = buscarMunicipio(pMunicipio);
    	m.inscribirCiudadano(pPuesto, pCiudadano);
        verificarInvariante( );
    }

    /**
     * Busca el municipio cuyo nombre es dado por parmetro. <br>
     * pre:		La lista de municipios se encuentra  inicializada
     * @param pNombreMunicipio Nombre del municipio. pNombreMunicipio != null && pNombreMunicipio != "".
     * @return Municipio con el nombre dado, null en caso que no lo encuentre.
     */
    public Municipio buscarMunicipio( String pNombreMunicipio )
    {
        Municipio municipioBuscado = null;
        for( int i = 0; i < municipios.size( ); i++ )
        {
            Municipio m = municipios.get( i );
            if( m.darNombre( ).equals( pNombreMunicipio ) )
            {
                municipioBuscado = m;
            }
        }
        return municipioBuscado;
    }

    /**
     * Calcula el nmero de inscritos del genero que llega por parmetro en el departamento.
     * pre:		La lista de municipios se encuentra inicializada
     * @param g el genero del que se desean conocer los inscritos
     * @return Nmero de inscritos del genero dado.
     */
    public int calcularNumeroInscritosPorGenero(Genero g )
    {
        int numInscritos = 0;
        for( int i = 0; i < municipios.size( ); i++ )
        {
            Municipio m = municipios.get( i );
            numInscritos += m.calcularNumeroInscritosPorGenero(g);
        }

        return numInscritos;
    }

    /**
     * Verifica si est inscrito o no un ciudadano en el departamento. <br>
     * pre:		La lista de municipios se encuentra inicializada
     * @param pCedulaCiudadano Cdula del ciudadano que se desea saber si est inscrito o no en el puesto de votacin. pCedulaCiudadano != null && pCedulaCiudadano != "".
     * @return True si el ciudadano se encuentra inscrito en algn municipio del departamento o false de lo contrario.
     */
    public boolean existeCiudadano( String pCedulaCiudadano )
    {
        boolean seEncuentra = false;
        for( int i = 0; i < municipios.size( ); i++ )
        {
            Municipio m = municipios.get( i );
            if( m.existeCiudadano( pCedulaCiudadano ) )
            {
                seEncuentra = true;
            }
        }
        return seEncuentra;
    }
    /**
     * Retorna la representacin en String del departamento.
     * @return Una cadena con los nombres con el siguiente formato: <nombre>.
     */
    public String toString( )
    {
        return nombre;
    }

    // -----------------------------------------------------------------
    // Invariante
    // -----------------------------------------------------------------

    /**
     * Verifica el invariante de la clase: <br>
     * <b>inv:</b><br>
     * nombre != null && nombre != "". <br>
     * municipios != null. <br>
     */
    private void verificarInvariante( )
    {
        assert nombre != null && !nombre.equals( "" ) : "El nombre del departamento no puede ser nulo o vaco.";
        assert municipios != null : "Los municipios del departamento no pueden ser nulos.";
    }
}
